@@ -1,6 +1,6 @@
 #!/usr/bin/env tclsh8.4
 # dpkgbuild.tcl
-# $Id: dpkgall.tcl 51780 2009-06-03 06:29:12Z jmr@macports.org $
+# $Id: dpkgall.tcl 68752 2010-06-12 13:57:23Z jmr@macports.org $
 #
 # Copyright (c) 2004 Landon Fuller <landonf@macports.org>
 # Copyright (c) 2003 Kevin Van Vechten <kevin@opendarwin.org>
@@ -347,7 +347,7 @@ proc main {argc argv} {
 
 	# If no portlist file was specified, create a portlist that includes all ports
 	if {[llength $portlist] == 0 || "$buildall_flag" == "true"} {
-		set res [mportsearch {.*}]
+		set res [mportlistall]
 		foreach {name array} $res {
 			lappend portlist $name
 		}
@@ -690,7 +690,7 @@ proc initialize_system {args} {
 	set variations ""
 
 	foreach port [get_required_ports] {
-		if {[catch {do_portexec $port [array get options] [array get variants] install} result]} {
+		if {[catch {do_portexec $port [array get options] [array get variants] activate} result]} {
 			global errorInfo
 			ui_debug "$errorInfo"
 			ui_noisy_error "Fatal error: $result"
@@ -704,7 +704,7 @@ proc initialize_system {args} {
 			set port [lindex $portlist 0]
 
 			ui_msg "Uninstalling $port."
-			if { [catch {portuninstall::uninstall $portname $portversion} result] } {
+			if { [catch {registry_uninstall::uninstall $portname $portversion} result] } {
 				global errorInfo
 				ui_debug "$errorInfo"
 				ui_noisy_errorr "Fatal error: Uninstalling $port failed: $result"
@@ -745,7 +745,7 @@ proc do_portexec {port options variants target} {
 		exit 1
 	}
 
-	if {[catch {set result [mportexec $workername install]} result] || $result == 1} {
+	if {[catch {set result [mportexec $workername $target]} result] || $result == 1} {
 
 		# Close the port
 		mportclose $workername
@@ -757,7 +757,7 @@ proc do_portexec {port options variants target} {
 
 proc get_portinfo {port} {
 	set searchstring [regex_escape_portname $port]
-	set res [mportsearch "^${searchstring}\$"]
+	set res [mportlookup ${searchstring}]
 
 	if {[llength $res] < 2} {
 		return -code error "Port \"$port\" not found in index."
