@@ -3,7 +3,7 @@
  * Copyright (c) 2005-2006 Paul Guyot <pguyot@kallisys.net>,
  * All rights reserved.
  *
- * $Id: darwintrace.c 50702 2009-05-07 08:59:20Z toby@macports.org $
+ * $Id: darwintrace.c 66640 2010-04-18 15:52:24Z raimue@macports.org $
  *
  * @APPLE_BSD_LICENSE_HEADER_START@
  * 
@@ -156,7 +156,16 @@ static char* __env_darwintrace_log;
 #endif
 #else
 #if DARWINTRACE_DEBUG_OUTPUT
-#define dprintf(format, param) fprintf(stderr, format, param)
+__attribute__ ((format (printf, 1, 2)))
+static inline
+int dprintf(const char *format, ...) {
+    int ret;
+    va_list args;
+    va_start(args, format);
+    ret = vfprintf(stderr, format, args);
+    va_end(args);
+    return ret;
+}
 #else
 #define dprintf(format, param)
 #endif
@@ -239,8 +248,10 @@ inline char* __darwintrace_alloc_env(const char* varName, const char* varValue)
 	if (varValue) {
 		int theSize = strlen(varName) + strlen(varValue) + 2;
 		theResult = (char*) malloc(theSize);
-		sprintf(theResult, "%s=%s", varName, varValue);
-		theResult[theSize - 1] = 0;
+		if (theResult) {
+		    snprintf(theResult, theSize, "%s=%s", varName, varValue);
+		    theResult[theSize - 1] = 0;
+		}
 	}
 	
 	return theResult;
@@ -250,6 +261,7 @@ inline char* __darwintrace_alloc_env(const char* varName, const char* varValue)
  * This function checks that envp contains the global variables we had when the
  * library was loaded and modifies it if it doesn't.
  */
+__attribute__((always_inline))
 inline char* const* __darwintrace_restore_env(char* const envp[]) {
 	/* allocate the strings. */
 	/* we don't care about the leak here because we're going to call execve,
@@ -334,14 +346,15 @@ inline void __darwintrace_setup() {
 			int sock=socket(AF_UNIX, SOCK_STREAM, 0);
 			struct sockaddr_un sun;
 			sun.sun_family=AF_UNIX;
-			strcpy(sun.sun_path, __env_darwintrace_log);
+			strncpy(sun.sun_path, __env_darwintrace_log, sizeof(sun.sun_path));
 			if(connect(sock, (struct sockaddr*)&sun, strlen(__env_darwintrace_log)+1+sizeof(sun.sun_family))!=-1)
 			{
 				dprintf("darwintrace: connect successful. socket %d\n", sock);
 				__darwintrace_fd=sock;
 				ask_for_filemap();
 			} else {
-				dprintf("connect failed %d :-(\n", errno);
+				dprintf("connect failed: %s\n", strerror(errno));
+				abort();
 			}
 			errno = olderrno;
 		}
@@ -474,8 +487,8 @@ static int ask_for_dependency(char * path)
 	if(is_directory(path))
 		return 1;
 	
-	strcpy(buffer, "dep_check\t");
-	strcpy(buffer+10, path);
+	strncpy(buffer, "dep_check\t", sizeof(buffer));
+	strncpy(buffer+10, path, sizeof(buffer)-10);
 	p=exchange_with_port(buffer, strlen(buffer)+1, 1, 0);
 	if(p==(char*)-1||!p)
 		return 0;
@@ -546,12 +559,14 @@ inline int __darwintrace_is_in_sandbox(const char* path, char * newpath) {
 		p=strdup(path);
 	else
 	{
-		p=(char*)malloc(BUFFER_SIZE);
-		(void) getcwd(p, BUFFER_SIZE-1);
-		_=p+strlen(p)+1;
-		if(_[-1]!='/')
-			*_++='/';
-		strncpy(_, path, BUFFER_SIZE-(_-p));
+		p=(char*)malloc(MAXPATHLEN);
+		if (getcwd(p, MAXPATHLEN-1) == NULL) {
+			fprintf(stderr, "darwintrace: getcwd: %s, path was: %s\n", strerror(errno), path);
+			abort();
+		}
+		if (p[strlen(p)-1] != '/')
+			strcat(p, "/");
+		strcat(p, path);
 	}
 	__darwintrace_cleanup_path(p);
 			
