@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portconfigure.tcl
-# $Id: portconfigure.tcl 55680 2009-08-17 03:53:54Z jmr@macports.org $
+# $Id: portconfigure.tcl 68781 2010-06-13 14:26:15Z jmr@macports.org $
 #
 # Copyright (c) 2002 - 2003 Apple Computer, Inc.
 # Copyright (c) 2007 Markus W. Weissmann <mww@macports.org>
@@ -36,7 +36,7 @@ package require portutil 1.0
 
 set org.macports.configure [target_new org.macports.configure portconfigure::configure_main]
 target_provides ${org.macports.configure} configure
-target_requires ${org.macports.configure} main fetch extract checksum patch
+target_requires ${org.macports.configure} main fetch checksum extract patch
 target_prerun ${org.macports.configure} portconfigure::configure_start
 
 namespace eval portconfigure {
@@ -48,6 +48,7 @@ commands configure autoreconf automake autoconf xmkmf
 default configure.env       ""
 default configure.pre_args  {--prefix=${prefix}}
 default configure.cmd       ./configure
+default configure.nice      {${buildnicevalue}}
 default configure.dir       {${worksrcpath}}
 default autoreconf.dir      {${worksrcpath}}
 default autoreconf.pre_args {--install}
@@ -151,18 +152,20 @@ default configure.bison             {}
 default configure.pkg_config        {}
 default configure.pkg_config_path   {}
 
-options configure.build_arch
-default configure.build_arch {${build_arch}}
+options configure.build_arch configure.ld_archflags configure.sdkroot
+default configure.build_arch {[portconfigure::choose_supported_archs ${build_arch}]}
+default configure.ld_archflags {[portconfigure::configure_get_ld_archflags]}
+default configure.sdkroot {[portconfigure::configure_get_sdkroot]}
 foreach tool {cc cxx objc f77 f90 fc} {
     options configure.${tool}_archflags
     default configure.${tool}_archflags  "\[portconfigure::configure_get_archflags $tool\]"
 }
 
 options configure.universal_archs configure.universal_args configure.universal_cflags configure.universal_cppflags configure.universal_cxxflags configure.universal_ldflags
-default configure.universal_archs       {${universal_archs}}
+default configure.universal_archs       {[portconfigure::choose_supported_archs ${universal_archs}]}
 default configure.universal_args        {--disable-dependency-tracking}
 default configure.universal_cflags      {[portconfigure::configure_get_universal_cflags]}
-default configure.universal_cppflags    {[portconfigure::configure_get_universal_cppflags]}
+default configure.universal_cppflags    {}
 default configure.universal_cxxflags    {[portconfigure::configure_get_universal_cflags]}
 default configure.universal_ldflags     {[portconfigure::configure_get_universal_ldflags]}
 
@@ -206,11 +209,38 @@ proc portconfigure::configure_start {args} {
         macports-gcc-4.2 { set name "MacPorts gcc 4.2" }
         macports-gcc-4.3 { set name "MacPorts gcc 4.3" }
         macports-gcc-4.4 { set name "MacPorts gcc 4.4" }
+        macports-gcc-4.5 { set name "MacPorts gcc 4.5" }
         default { return -code error "Invalid value for configure.compiler" }
     }
     ui_debug "Using compiler '$name'"
 }
 
+# internal function to choose the default configure.build_arch and
+# configure.universal_archs based on supported_archs and build_arch or
+# universal_archs
+proc portconfigure::choose_supported_archs {archs} {
+    global supported_archs
+    if {$supported_archs == ""} {
+        return $archs
+    }
+    set ret {}
+    foreach arch $archs {
+        if {[lsearch -exact $supported_archs $arch] != -1} {
+            set add_arch $arch
+        } elseif {$arch == "x86_64" && [lsearch -exact $supported_archs "i386"] != -1} {
+            set add_arch "i386"
+        } elseif {$arch == "ppc64" && [lsearch -exact $supported_archs "ppc"] != -1} {
+            set add_arch "ppc"
+        } else {
+            continue
+        }
+        if {[lsearch -exact $ret $add_arch] == -1} {
+            lappend ret $add_arch
+        }
+    }
+    return $ret
+}
+
 # internal function to determine the compiler flags to select an arch
 proc portconfigure::configure_get_archflags {tool} {
     global configure.build_arch configure.m32 configure.m64 configure.compiler
@@ -231,6 +261,35 @@ proc portconfigure::configure_get_archflags {tool} {
     return $flags
 }
 
+# internal function to determine the ld flags to select an arch
+# Unfortunately there's no consistent way to do this when the compiler
+# doesn't support -arch, because it could be used to link rather than using
+# ld directly. So we punt and let portfiles deal with that case.
+proc portconfigure::configure_get_ld_archflags {args} {
+    global configure.build_arch
+    if {${configure.build_arch} != "" && [arch_flag_supported]} {
+        return "-arch ${configure.build_arch}"
+    } else {
+        return ""
+    }
+}
+
+proc portconfigure::configure_get_sdkroot {} {
+    global developer_dir macosx_deployment_target macosx_version os.arch os.platform
+    if {${os.platform} == "darwin" && ($macosx_deployment_target != $macosx_version
+        || (${os.arch} == "powerpc" && $macosx_version == "10.4" && [variant_exists universal] && [variant_isset universal]))} {
+        if {$macosx_deployment_target == "10.4"} {
+            set sdk "${developer_dir}/SDKs/MacOSX10.4u.sdk"
+        } else {
+            set sdk "${developer_dir}/SDKs/MacOSX${macosx_deployment_target}.sdk"
+        }
+        if {[file exists $sdk]} {
+            return $sdk
+        }
+    }
+    return ""
+}
+
 # internal function to determine the "-arch xy" flags for the compiler
 proc portconfigure::configure_get_universal_archflags {args} {
     global configure.universal_archs
@@ -245,37 +304,14 @@ proc portconfigure::configure_get_universal_archflags {args} {
     return $flags
 }
 
-# internal function to determine the CPPFLAGS for the compiler
-proc portconfigure::configure_get_universal_cppflags {args} {
-    global os.arch os.major developer_dir
-    set flags ""
-    # include sysroot in CPPFLAGS too (twice), for the benefit of autoconf
-    if {${os.arch} == "powerpc" && ${os.major} == "8"} {
-        set flags "-isysroot ${developer_dir}/SDKs/MacOSX10.4u.sdk"
-    }
-    return $flags
-}
-
 # internal function to determine the CFLAGS for the compiler
 proc portconfigure::configure_get_universal_cflags {args} {
-    global os.arch os.major developer_dir
-    set flags [configure_get_universal_archflags]
-    # these flags should be valid for C/C++ and similar compiler frontends
-    if {${os.arch} == "powerpc" && ${os.major} == "8"} {
-        set flags "-isysroot ${developer_dir}/SDKs/MacOSX10.4u.sdk ${flags}"
-    }
-    return $flags
+    return [configure_get_universal_archflags]
 }
 
 # internal function to determine the LDFLAGS for the compiler
 proc portconfigure::configure_get_universal_ldflags {args} {
-    global os.arch os.major developer_dir
-    set flags [configure_get_universal_archflags]
-    # works around linking without using the CFLAGS, outside of automake
-    if {${os.arch} == "powerpc" && ${os.major} == "8"} {
-        set flags "-Wl,-syslibroot,${developer_dir}/SDKs/MacOSX10.4u.sdk ${flags}"
-    }
-    return $flags
+    return [configure_get_universal_archflags]
 }
 
 # internal proc to determine if the compiler supports -arch
@@ -298,16 +334,14 @@ proc portconfigure::arch_flag_supported {args} {
 
 # internal function to determine the default compiler
 proc portconfigure::configure_get_default_compiler {args} {
-    global os.platform os.major
-    set compiler ""
-    switch -exact "${os.platform} ${os.major}" {
-        "darwin 7" { set compiler gcc-3.3 }
-        "darwin 8" { set compiler gcc-4.0 }
-        "darwin 9" { set compiler gcc-4.0 }
-        "darwin 10" { set compiler gcc-4.2 }
-        default { set compiler gcc }
+    global macosx_deployment_target
+    switch -exact ${macosx_deployment_target} {
+        "10.4"      -
+        "10.5"      { return gcc-4.0 }
+        "10.6"      { return gcc-4.2 }
+        "10.7"      { return llvm-gcc-4.2 }
+        default     { return gcc }
     }
-    return $compiler
 }
 
 # internal function to find correct compilers
@@ -349,9 +383,10 @@ proc portconfigure::configure_get_compiler {type} {
         }
         llvm-gcc-4.2 {
             switch -exact ${type} {
-                cc   { set ret ${developer_dir}/usr/llvm-gcc-4.2/bin/llvm-gcc-4.2 }
-                objc { set ret ${developer_dir}/usr/llvm-gcc-4.2/bin/llvm-gcc-4.2 }
-                cxx  { set ret ${developer_dir}/usr/llvm-gcc-4.2/bin/llvm-g++-4.2 }
+                cc   { set ret ${developer_dir}/usr/bin/llvm-gcc-4.2 }
+                objc { set ret ${developer_dir}/usr/bin/llvm-gcc-4.2 }
+                cxx  { set ret ${developer_dir}/usr/bin/llvm-g++-4.2 }
+                cpp  { set ret ${developer_dir}/usr/bin/llvm-cpp-4.2 }
             }
         }
         clang {
@@ -449,6 +484,17 @@ proc portconfigure::configure_get_compiler {type} {
                 f90  { set ret ${prefix}/bin/gfortran-mp-4.4 }
             }
         }
+        macports-gcc-4.5 {
+            switch -exact ${type} {
+                cc   { set ret ${prefix}/bin/gcc-mp-4.5 }
+                objc { set ret ${prefix}/bin/gcc-mp-4.5 }
+                cxx  { set ret ${prefix}/bin/g++-mp-4.5 }
+                cpp  { set ret ${prefix}/bin/cpp-mp-4.5 }
+                fc   { set ret ${prefix}/bin/gfortran-mp-4.5 }
+                f77  { set ret ${prefix}/bin/gfortran-mp-4.5 }
+                f90  { set ret ${prefix}/bin/gfortran-mp-4.5 }
+            }
+        }
     }
     return $ret
 }
@@ -458,8 +504,8 @@ proc portconfigure::configure_main {args} {
     global worksrcpath use_configure use_autoreconf use_autoconf use_automake use_xmkmf
     global configure.env configure.pipe configure.libs configure.classpath configure.universal_args
     global configure.perl configure.python configure.ruby configure.install configure.awk configure.bison configure.pkg_config configure.pkg_config_path
-    global configure.ccache configure.distcc configure.cpp configure.javac configure.march configure.mtune
-    foreach tool {cc cxx objc f77 f90 fc} {
+    global configure.ccache configure.distcc configure.cpp configure.javac configure.march configure.mtune configure.sdkroot
+    foreach tool {cc cxx objc f77 f90 fc ld} {
         global configure.${tool} configure.${tool}_archflags
     }
     foreach flags {cflags cppflags cxxflags objcflags ldflags fflags f90flags fcflags} {
@@ -541,7 +587,15 @@ proc portconfigure::configure_main {args} {
         append_list_to_environment_value configure "BISON" ${configure.bison}
         append_list_to_environment_value configure "PKG_CONFIG" ${configure.pkg_config}
         append_list_to_environment_value configure "PKG_CONFIG_PATH" ${configure.pkg_config_path}
-        
+
+        # add SDK flags if cross-compiling (or universal on ppc tiger)
+        if {${configure.sdkroot} != ""} {
+            foreach flags {CPPFLAGS CFLAGS CXXFLAGS OBJCFLAGS} {
+                append_list_to_environment_value configure $flags "-isysroot ${configure.sdkroot}"
+            }
+            append_list_to_environment_value configure "LDFLAGS" "-Wl,-syslibroot,${configure.sdkroot}"
+        }
+
         # add extra flags that are conditional on whether we're building universal
         if {[variant_exists universal] && [variant_isset universal]} {
             foreach flags {CFLAGS OBJCFLAGS} {
@@ -552,7 +606,7 @@ proc portconfigure::configure_main {args} {
             append_list_to_environment_value configure "LDFLAGS" ${configure.universal_ldflags}
             eval configure.pre_args-append ${configure.universal_args}
         } else {
-            foreach {tool flags} {cc CFLAGS cxx CXXFLAGS objc OBJCFLAGS f77 FFLAGS f90 F90FLAGS fc FCFLAGS} {
+            foreach {tool flags} {cc CFLAGS cxx CXXFLAGS objc OBJCFLAGS f77 FFLAGS f90 F90FLAGS fc FCFLAGS ld LDFLAGS} {
                 append_list_to_environment_value configure $flags [set configure.${tool}_archflags]
                 if {${configure.march} != {}} {
                     append_list_to_environment_value configure $flags "-march=${configure.march}"
