@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portutil.tcl
-# $Id: portutil.tcl 55829 2009-08-20 02:21:26Z raimue@macports.org $
+# $Id: portutil.tcl 68692 2010-06-10 12:45:51Z jmr@macports.org $
 #
 # Copyright (c) 2004 Robert Shaw <rshaw@opendarwin.org>
 # Copyright (c) 2002 Apple Computer, Inc.
@@ -289,7 +289,7 @@ proc option_proc_trace {optionName index op} {
 # and used to form a standard set of command options.
 proc commands {args} {
     foreach option $args {
-        options use_${option} ${option}.dir ${option}.pre_args ${option}.args ${option}.post_args ${option}.env ${option}.type ${option}.cmd
+        options use_${option} ${option}.dir ${option}.pre_args ${option}.args ${option}.post_args ${option}.env ${option}.nice ${option}.type ${option}.cmd
     }
 }
 
@@ -328,14 +328,14 @@ proc command_string {command} {
 # command_prefix    additional command prefix (typically pipe command)
 # command_suffix    additional command suffix (typically redirection)
 proc command_exec {command args} {
-    global ${command}.env ${command}.env_array env
-    set notty 0
+    global ${command}.env ${command}.env_array ${command}.nice env macosx_version
+    set notty ""
     set command_prefix ""
     set command_suffix ""
 
     if {[llength $args] > 0} {
         if {[lindex $args 0] == "-notty"} {
-            set notty 1
+            set notty "-notty"
             set args [lrange $args 1 end]
         }
 
@@ -358,10 +358,24 @@ proc command_exec {command args} {
     if {[option macosx_deployment_target] ne ""} {
         set ${command}.env_array(MACOSX_DEPLOYMENT_TARGET) [option macosx_deployment_target]
     }
+    set ${command}.env_array(CC_PRINT_OPTIONS) "YES"
+    set ${command}.env_array(CC_PRINT_OPTIONS_FILE) [file join [option workpath] ".CC_PRINT_OPTIONS"]
+    if {[option compiler.cpath] ne ""} {
+        set ${command}.env_array(CPATH) [join [option compiler.cpath] :]
+    }
+    if {[option compiler.library_path] ne ""} {
+        set ${command}.env_array(LIBRARY_PATH) [join [option compiler.library_path] :]
+    }
 
     # Debug that.
     ui_debug "Environment: [environment_array_to_string ${command}.env_array]"
 
+    # Prepare nice value change
+    set nice ""
+    if {[info exists ${command}.nice] && [set ${command}.nice] != ""} {
+        set nice "-nice [set ${command}.nice]"
+    }
+
     # Get the command string.
     set cmdstring [command_string ${command}]
 
@@ -373,17 +387,16 @@ proc command_exec {command args} {
     array set env [array get ${command}.env_array]
     # Call the command.
     set fullcmdstring "$command_prefix $cmdstring $command_suffix"
-    if {$notty} {
-        set code [catch {system -notty $fullcmdstring} result]
-    } else {
-        set code [catch {system $fullcmdstring} result]
-    }
+    set code [catch {eval system $notty $nice \$fullcmdstring} result]
+
     # Unset the command array until next time.
     array unset ${command}.env_array
 
     # Restore the environment.
     array unset env *
-    unsetenv *
+    if {$macosx_version == "10.5"} {
+        unsetenv *
+    }
     array set env [array get saved_env]
 
     # Return as if system had been called directly.
@@ -436,13 +449,46 @@ proc default_check {optionName index op} {
     }
 }
 
-# Notes are displayed at (1) the end of the activation phase and (2) when
-# action_notes is executed.
-proc notes {args} {
-    global PortInfo portnotes
+##
+# Filter options which take strings removing indent to ease Portfile writing
+proc handle_option_string {option action args} {
+    global $option
 
-    set PortInfo(notes) [string trim [join $args]]
-    set portnotes $PortInfo(notes)
+    switch $action {
+        set {
+            set args [join $args]
+
+            set fulllist {}
+            # args is a list of strings/list
+            foreach arg $args {
+                # Strip trailing empty lines
+                if {[string index $arg 0] == "\n"} {
+                    set arg [string range $arg 1 end]
+                }
+                if {[string index $arg end] == "\n"} {
+                    set arg [string range $arg 0 end-1]
+                }
+
+                # Determine indent level
+                set indent ""
+                for {set i 0} {$i < [string length $arg]} {incr i} {
+                    set c [string index $arg $i]
+                    if {$c != " " && $c != "\t"} {
+                        break
+                    }
+                    append indent $c
+                }
+                # Remove indent on first line
+                set arg [string replace $arg 0 [expr $i - 1]]
+                # Remove indent on each other line
+                set arg [string map "\"\n$indent\" \"\n\"" $arg]
+
+                lappend fulllist $arg
+            }
+
+            set $option $fulllist
+        }
+    }
 }
 
 # variant <provides> [<provides> ...] [requires <requires> [<requires>]]
@@ -539,6 +585,13 @@ proc variant {args} {
     array set vinfo [list $variant_provides [array get variant]]
     set PortInfo(vinfo) [array get vinfo]
 
+    if {[variant_isset $variant_provides]} {
+        # set variants that this one requires
+        foreach req [ditem_key $ditem requires] {
+            variant_set $req
+        }
+    }
+
     # Finally append the ditem to the dlist.
     lappend all_variants $ditem
 }
@@ -648,23 +701,21 @@ proc variant_desc {porturl variant} {
     }
 }
 
-# platform <os> [<release>] [<arch>]
+# platform [<os> [<release>]] [<arch>]
 # Portfile level procedure to provide support for declaring platform-specifics
-# Basically, just wrap 'variant', so that Portfiles' platform declarations can
+# Basically, just a fancy 'if', so that Portfiles' platform declarations can
 # be more readable, and support arch and version specifics
 proc platform {args} {
-    global all_variants PortInfo os.platform os.arch os.version os.major
+    global os.platform os.subplatform os.arch os.major
 
     set len [llength $args]
     if {$len < 2} {
-        return -code error "Malformed platform variant specification"
+        return -code error "Malformed platform specification"
     }
     set code [lindex $args end]
     set os [lindex $args 0]
     set args [lrange $args 1 [expr $len - 2]]
 
-    set ditem [variant_new "temp-variant"]
-
     foreach arg $args {
         if {[regexp {(^[0-9]+$)} $arg match result]} {
             set release $result
@@ -673,43 +724,19 @@ proc platform {args} {
         }
     }
 
-    # Add the variant for this platform
-    set platform $os
-    if {[info exists release]} { set platform ${platform}_${release} }
-    if {[info exists arch]} { set platform ${platform}_${arch} }
-
-    # Pick up a unique name.
-    if {[variant_exists $platform]} {
-        set suffix 1
-        while {[variant_exists "${platform}_${suffix}"]} {
-            incr suffix
-        }
-
-        set platform "${platform}_${suffix}"
+    set match 0
+    # 'os' could be a platform or an arch when it's alone
+    if {$len == 2 && ($os == ${os.platform} || $os == ${os.subplatform} || $os == ${os.arch})} {
+        set match 1
+    } elseif {($os == ${os.platform} || $os == ${os.subplatform})
+              && (![info exists release] || ${os.major} == $release)
+              && (![info exists arch] || ${os.arch} == $arch)} {
+        set match 1
     }
-    variant $platform $code
 
-    # Set the variant if this platform matches the platform we're on
-    set matches 1
-    if {[info exists os.platform] && ${os.platform} == $os} {
-        set sel_platform $os
-        if {[info exists os.major] && [info exists release]} {
-            if {${os.major} == $release } {
-                set sel_platform ${sel_platform}_${release}
-            } else {
-                set matches 0
-            }
-        }
-        if {$matches == 1 && [info exists arch] && [info exists os.arch]} {
-            if {${os.arch} == $arch} {
-                set sel_platform ${sel_platform}_${arch}
-            } else {
-                set matches 0
-            }
-        }
-        if {$matches == 1} {
-            variant_set $sel_platform
-        }
+    # Execute the code if this platform matches the platform we're on
+    if {$match} {
+        uplevel 1 $code
     }
 }
 
@@ -835,7 +862,7 @@ proc reinplace {args}  {
         set arg [lindex $args 0]
         if {[string index $arg 0] eq "-"} {
             set args [lrange $args 1 end]
-            switch [string range $arg 1 end] {
+            switch -- [string range $arg 1 end] {
                 E {
                     set extended 1
                 }
@@ -1109,32 +1136,6 @@ proc ln {args} {
     return
 }
 
-# filefindbypath
-# Provides searching of the standard path for included files
-proc filefindbypath {fname} {
-    global distpath filesdir worksrcdir portpath
-
-    if {[file readable $portpath/$fname]} {
-        return $portpath/$fname
-    } elseif {[file readable $portpath/$filesdir/$fname]} {
-        return $portpath/$filesdir/$fname
-    } elseif {[file readable $distpath/$fname]} {
-        return $distpath/$fname
-    }
-    return ""
-}
-
-# include
-# Source a file, looking for it along a standard search path.
-proc include {fname} {
-    set tgt [filefindbypath $fname]
-    if {[string length $tgt]} {
-        uplevel "source $tgt"
-    } else {
-        return -code error "Unable to find include file $fname"
-    }
-}
-
 # makeuserproc
 # This procedure re-writes the user-defined custom target to include
 # all the globals in its scope.  This is undeniably ugly, but I haven't
@@ -1207,7 +1208,7 @@ global ports_dry_last_skipped
 set ports_dry_last_skipped ""
 
 proc target_run {ditem} {
-    global target_state_fd workpath ports_trace PortInfo ports_dryrun ports_dry_last_skipped
+    global target_state_fd workpath portpath ports_trace PortInfo ports_dryrun ports_dry_last_skipped worksrcpath prefix
     set portname [option name]
     set result 0
     set skipped 0
@@ -1258,7 +1259,7 @@ proc target_run {ditem} {
                 if {[ditem_contains $ditem prerun]} {
                     set result [catch {[ditem_key $ditem prerun] $targetname} errstr]
                 }
-
+                set_phase  $target
                 #start tracelib
                 if {($result ==0
                   && [info exists ports_trace]
@@ -1368,6 +1369,18 @@ proc target_run {ditem} {
                 }
             }
         }
+        if {[exists copy_log_files]} {
+            set log_files [option copy_log_files]
+            set log_dir [getportlogpath $portpath]
+            file mkdir $log_dir
+ 
+            foreach log_file $log_files {
+                set from "$worksrcpath/$log_file"
+                if {[file exists $from]} {
+                    file copy -force $from $log_dir
+                }
+            }
+        }
         if {$result == 0} {
             # Only write to state file if:
             # - we indeed performed this step.
@@ -1436,29 +1449,25 @@ proc recursive_collect_deps {portname deptypes {depsfound {}}} \
 
 
 proc eval_targets {target} {
-    global targets target_state_fd name version revision portvariants ports_dryrun user_options
+    global targets target_state_fd name version revision portvariants epoch ports_dryrun user_options
     set dlist $targets
 
     # the statefile will likely be autocleaned away after install,
-    # so special-case ignore already-completed install and activate
+    # so special-case already-completed install and activate
     if {[registry_exists $name $version $revision $portvariants]} {
         if {$target == "install"} {
             ui_debug "Skipping $target ($name) since this port is already installed"
             return 0
         } elseif {$target == "activate"} {
-            set regref [registry_open $name $version $revision $portvariants]
+            set regref [registry_open $name $version $revision $portvariants $epoch]
             if {[registry_prop_retr $regref active] != 0} {
                 # Something to close the registry entry may be called here, if it existed.
                 ui_debug "Skipping $target ($name @${version}_${revision}${portvariants}) since this port is already active"
+                return 0
             } else {
-                # do the activate here since target_run doesn't know how to selectively ignore the preceding steps
-                if {[info exists ports_dryrun] && $ports_dryrun == "yes"} {
-                    ui_msg "For $name: skipping $target (dry run)"
-                } else {
-                    registry_activate $name ${version}_${revision}${portvariants} [array get user_options]
-                }
+                # run the activate target but ignore its (completed) dependencies
+                return [target_run [lindex [dlist_search $dlist provides $target] 0]]
             }
-            return 0
         }
     }
 
@@ -1495,31 +1504,24 @@ proc eval_targets {target} {
 # open_statefile
 # open file to store name of completed targets
 proc open_statefile {args} {
-    global workpath worksymlink place_worksymlink name portpath ports_ignore_older
+    global workpath worksymlink place_worksymlink name portpath ports_ignore_older ports_dryrun
     global usealtworkpath altprefix env applications_dir portbuildpath
 
-    if {![file isdirectory $workpath]} {
-        file mkdir $workpath
-        chownAsRoot $portbuildpath
-    }
-    
-    if { [getuid] != 0 } {
-        ui_msg "MacPorts running without privileges.\
-                You may be unable to complete certain actions (eg install)."
-    }
-    
     # de-escalate privileges if MacPorts was started with sudo
     dropPrivileges
-    
+
     if {$usealtworkpath} {
-        set newsourcepath "$altprefix/$portpath"
+         ui_warn_once "privileges" "MacPorts running without privileges.\
+                You may be unable to complete certain actions (e.g. install)."
+
+        set newsourcepath "${altprefix}${portpath}"
     
         # copy Portfile (and patch files) if not there already
         # note to maintainers/devs: the original portfile in /opt/local is ALWAYS the one that will be
         #    read by macports. The copying of the portfile is done to preserve the symlink provided
         #    historically by macports from the portfile directory to the work directory.
         #    It is NOT read by MacPorts.
-        if {![file exists ${newsourcepath}/Portfile] } {
+        if {![file exists ${newsourcepath}/Portfile] && ![tbool ports_dryrun]} {
             file mkdir $newsourcepath
             ui_debug "$newsourcepath created"
             ui_debug "Going to copy: ${portpath}/Portfile"
@@ -1531,14 +1533,27 @@ proc open_statefile {args} {
         }
     }
 
+    if {![file isdirectory $workpath] && ![tbool ports_dryrun]} {
+        file mkdir $workpath
+        chownAsRoot $portbuildpath
+        # Create a symlink to the workpath for port authors
+        if {[tbool place_worksymlink] && ![file isdirectory $worksymlink]} {
+            ui_debug "Attempting ln -sf $workpath $worksymlink"
+            ln -sf $workpath $worksymlink
+        }
+    }
+
     # flock Portfile
     set statefile [file join $workpath .macports.${name}.state]
     if {[file exists $statefile]} {
-        if {![file writable $statefile]} {
+        if {![file writable $statefile] && ![tbool ports_dryrun]} {
             return -code error "$statefile is not writable - check permission on port directory"
         }
+        if {[file mtime ${portpath}/Portfile] >= [clock seconds]} {
+            return -code error "Portfile is from the future - check date and time of your system"
+        }
         if {!([info exists ports_ignore_older] && $ports_ignore_older == "yes") && [file mtime $statefile] < [file mtime ${portpath}/Portfile]} {
-            if {!([info exists ports_dryrun] && $ports_dryrun == "yes")} {
+            if {![tbool ports_dryrun]} {
                 ui_msg "Portfile changed since last build; discarding previous state."
                 delete $workpath
                 file mkdir $workpath
@@ -1546,26 +1561,24 @@ proc open_statefile {args} {
                 ui_msg "Portfile changed since last build but not discarding previous state (dry run)"
             }
         }
-    }
-
-    # Create a symlink to the workpath for port authors
-    if {[tbool place_worksymlink] && ![file isdirectory $worksymlink]} {
-        ui_debug "Attempting ln -sf $workpath $worksymlink"
-        ln -sf $workpath $worksymlink
+    } elseif {[tbool ports_dryrun]} {
+        set statefile /dev/null
     }
 
     set fd [open $statefile a+]
-    if {[catch {flock $fd -exclusive -noblock} result]} {
-        if {"$result" == "EAGAIN"} {
-            ui_msg "Waiting for lock on $statefile"
-        } elseif {"$result" == "EOPNOTSUPP"} {
-            # Locking not supported, just return
-            return $fd
-        } else {
-            return -code error "$result obtaining lock on $statefile"
+    if {![tbool ports_dryrun]} {
+        if {[catch {flock $fd -exclusive -noblock} result]} {
+            if {"$result" == "EAGAIN"} {
+                ui_msg "Waiting for lock on $statefile"
+            } elseif {"$result" == "EOPNOTSUPP"} {
+                # Locking not supported, just return
+                return $fd
+            } else {
+                return -code error "$result obtaining lock on $statefile"
+            }
         }
+        flock $fd -exclusive
     }
-    flock $fd -exclusive
     return $fd
 }
 
@@ -1641,6 +1654,7 @@ proc choose_variants {dlist variations} {
     upvar $variations upvariations
 
     set selected [list]
+    set negated [list]
 
     foreach ditem $dlist {
         # Enumerate through the provides, tallying the pros and cons.
@@ -1659,13 +1673,14 @@ proc choose_variants {dlist variations} {
             }
         }
 
-        if {$cons > 0} { continue }
-
+        if {$cons > 0} {
+            lappend negated $ditem
+        }
         if {$pros > 0 && $ignored == 0} {
             lappend selected $ditem
         }
     }
-    return $selected
+    return [list $selected $negated]
 }
 
 proc variant_run {ditem} {
@@ -1696,24 +1711,26 @@ proc variant_run {ditem} {
     # was turned on or off, a particular instance of the port is uniquely
     # characterized by the set of variants that are *on*. Thus, record those
     # variants in a string in a standard order as +var1+var2 etc.
-    # XXX: this doesn't quite work because of default variants, see ticket #2377
-proc canonicalize_variants {variants} {
+    # Can also do the same for -variants, for recording the negated list.
+proc canonicalize_variants {variants {sign "+"}} {
     array set vara $variants
     set result ""
     set vlist [lsort -ascii [array names vara]]
     foreach v $vlist {
-        if {$vara($v) == "+"} {
-            append result +$v
+        if {$vara($v) == $sign} {
+            append result "${sign}${v}"
         }
     }
     return $result
 }
 
 proc eval_variants {variations} {
-    global all_variants ports_force PortInfo portvariants
+    global all_variants ports_force PortInfo portvariants negated_variants
     set dlist $all_variants
     upvar $variations upvariations
     set chosen [choose_variants $dlist upvariations]
+    set negated [lindex $chosen 1]
+    set chosen [lindex $chosen 0]
     set portname $PortInfo(name)
 
     # Check to make sure the requested variations are available with this
@@ -1751,7 +1768,7 @@ proc eval_variants {variations} {
     # it's convenient to check for inconsistent requests for
     # variations, namely foo +requirer -required where the 'requirer'
     # variant requires the 'required' one.
-    array set activevariants [list]
+    set activevariants [list]
     foreach dvar $newlist {
         set thevar [ditem_key $dvar provides]
         if {[info exists upvariations($thevar)] && $upvariations($thevar) eq "-"} {
@@ -1762,33 +1779,31 @@ proc eval_variants {variations} {
             ui_error "Inconsistent variant specification: $portname variant +$thevar is required by at least one of $chosenlist, but specified -$thevar"
             return 1
         }
-        set activevariants($thevar) "+"
+        lappend activevariants $thevar "+"
     }
 
     # Record a canonical variant string, used e.g. in accessing the registry
-    set portvariants [canonicalize_variants [array get activevariants]]
+    set portvariants [canonicalize_variants $activevariants]
 
     # Make this important information visible in PortInfo
-    set PortInfo(active_variants) [array get activevariants]
+    set PortInfo(active_variants) $activevariants
     set PortInfo(canonical_active_variants) $portvariants
 
-    # XXX: I suspect it would actually work better in the following
-    # block to record the activevariants in the statefile rather than
-    # the upvariations, since as far as I can see different sets of
-    # upvariations which amount to the same activevariants in the end
-    # can share all aspects of the build. But I'm leaving this alone
-    # for the time being, so that someone with more extensive
-    # experience can examine the idea before putting it into
-    # action. -- GlenWhitney
+    # now set the negated variants
+    set negated_list [list]
+    foreach dvar $negated {
+        set thevar [ditem_key $dvar provides]
+        lappend negated_list $thevar "-"
+    }
+    set negated_variants [canonicalize_variants $negated_list "-"]
 
     return 0
 }
 
-proc check_variants {variations target} {
+proc check_variants {target} {
     global targets ports_force ports_dryrun PortInfo
-    upvar $variations upvariations
     set result 0
-    set portname $PortInfo(name)
+    array set variations $PortInfo(active_variants)
 
     # Make sure the variations match those stored in the statefile.
     # If they don't match, print an error indicating a 'port clean'
@@ -1800,6 +1815,9 @@ proc check_variants {variations target} {
     # Assume we do not need the statefile
     set statereq 0
     set ditems [dlist_search $targets provides $target]
+    if {[llength $ditems] > 0} {
+        set ditems [dlist_append_dependents $targets [lindex $ditems 0] [list]]
+    }
     foreach d $ditems {
         if {[ditem_key $d state] != "no"} {
             # At least one matching target requires the state file
@@ -1813,13 +1831,13 @@ proc check_variants {variations target} {
         set state_fd [open_statefile]
 
         array set oldvariations {}
-        if {[check_statefile_variants upvariations oldvariations $state_fd]} {
-            ui_error "Requested variants \"[canonicalize_variants [array get upvariations]]\" do not match original selection \"[canonicalize_variants [array get oldvariations]]\".\nPlease use the same variants again, perform 'port clean $portname' or specify the force option (-f)."
+        if {[check_statefile_variants variations oldvariations $state_fd]} {
+            ui_error "Requested variants \"[canonicalize_variants [array get variations]]\" do not match original selection \"[canonicalize_variants [array get oldvariations]]\".\nPlease use the same variants again, perform 'port clean [option name]' or specify the force option (-f)."
             set result 1
         } elseif {!([info exists ports_dryrun] && $ports_dryrun == "yes")} {
             # Write variations out to the statefile
-            foreach key [array names upvariations *] {
-            write_statefile variant $upvariations($key)$key $state_fd
+            foreach key [array names variations *] {
+                write_statefile variant $variations($key)$key $state_fd
             }
         }
 
@@ -1834,14 +1852,13 @@ proc universal_setup {args} {
     if {[variant_exists universal]} {
         ui_debug "universal variant already exists, so not adding the default one"
     } elseif {[exists universal_variant] && ![option universal_variant]} {
-        ui_debug "'universal_variant no' specified, so not adding the default universal variant"
+        ui_debug "universal_variant is false, so not adding the default universal variant"
     } elseif {[exists use_xmkmf] && [option use_xmkmf]} {
         ui_debug "using xmkmf, so not adding the default universal variant"
-    } elseif {[exists use_configure] && ![option use_configure] && ![exists xcode.project]} {
-        # Allow +universal if port uses xcode portgroup.
-        ui_debug "not using configure, so not adding the default universal variant"
     } elseif {![exists os.universal_supported] || ![option os.universal_supported]} {
         ui_debug "OS doesn't support universal builds, so not adding the default universal variant"
+    } elseif {[llength [option supported_archs]] == 1} {
+        ui_debug "only one arch supported, so not adding the default universal variant"
     } else {
         ui_debug "adding the default universal variant"
         variant universal {}
@@ -2000,6 +2017,13 @@ proc handle_default_variants {option action {value ""}} {
 
 proc adduser {name args} {
     global os.platform
+
+    if {[getuid] != 0} {
+        ui_warn "adduser only works when running as root."
+        ui_warn "The requested user '$name' was not created."
+        return
+    }
+
     set passwd {*}
     set uid [nextuid]
     set gid [existsgroup nogroup]
@@ -2027,13 +2051,20 @@ proc adduser {name args} {
         exec $dscl . -create /Users/${name} UserShell ${shell}
     } else {
         # XXX adduser is only available for darwin, add more support here
-        ui_warn "WARNING: adduser is not implemented on ${os.platform}."
-        ui_warn "The requested user was not created."
+        ui_warn "adduser is not implemented on ${os.platform}."
+        ui_warn "The requested user '$name' was not created."
     }
 }
 
 proc addgroup {name args} {
     global os.platform
+
+    if {[getuid] != 0} {
+        ui_warn "addgroup only works when running as root."
+        ui_warn "The requested group '$name' was not created."
+        return
+    }
+
     set gid [nextgid]
     set realname ${name}
     set passwd {*}
@@ -2059,7 +2090,7 @@ proc addgroup {name args} {
         }
     } else {
         # XXX addgroup is only available for darwin, add more support here
-        ui_warn "WARNING: addgroup is not implemented on ${os.platform}."
+        ui_warn "addgroup is not implemented on ${os.platform}."
         ui_warn "The requested group was not created."
     }
 }
@@ -2181,12 +2212,11 @@ proc archiveTypeIsSupported {type} {
 # e.g. 'merge_lipo ${workpath}/pre-dest ${destroot} ${prefix}/bin/pstree i386 ppc
 # will merge binary files with lipo which have to be in the same (relative) path
 proc merge_lipo {base target file archs} {
-    set exec-lipo ""
+    set exec-lipo [list [findBinary lipo $portutil::autoconf::lipo_path]]
     foreach arch ${archs} {
-        set exec-lipo [concat ${exec-lipo} [list "-arch" "${arch}" "${base}/${arch}${file}"]]
+        lappend exec-lipo -arch ${arch} ${base}/${arch}${file}
     }
-    set exec-lipo [concat ${exec-lipo}]
-    system "[findBinary lipo $portutil::autoconf::lipo_path] ${exec-lipo} -create -output ${target}${file}"
+    eval exec ${exec-lipo} [list -create -output ${target}${file}]
 }
 
 # private function
@@ -2231,6 +2261,9 @@ proc merge {base} {
             set base_arch ${arch}
         }
     }
+    if {"" == ${base_arch}} {
+        return -code error [format [msgcat::mc "Cannot merge because directory '%s' contains no architecture directories."] ${base}]
+    }
     ui_debug "merging architectures ${archs}, base_arch is ${base_arch}"
 
     # traverse the base-architecture directory
@@ -2239,30 +2272,34 @@ proc merge {base} {
         set fpath [string range "${file}" [string length "${basepath}"] [string length "${file}"]]
         if {${fpath} != ""} {
             # determine the type (dir/file/link)
-            set filetype [exec [findBinary file $portutil::autoconf::file_path] "-b" "${basepath}${fpath}"]
-            switch -regexp ${filetype} {
+            switch [file type ${basepath}${fpath}] {
                 directory {
                     # just create directories
                     ui_debug "mrg: directory ${fpath}"
                     file mkdir "${destroot}${fpath}"
                 }
-                symbolic\ link.* {
+                link {
                     # copy symlinks, TODO: check if targets match!
                     ui_debug "mrg: symlink ${fpath}"
                     file copy "${basepath}${fpath}" "${destroot}${fpath}"
                 }
-                Mach-O.* {
-                    merge_lipo "${base}" "${destroot}" "${fpath}" "${archs}"
-                }
-                current\ ar\ archive {
-                    merge_lipo "${base}" "${destroot}" "${fpath}" "${archs}"
-                }
-                ASCII\ C\ program\ text {
-                    merge_cpp "${base}" "${destroot}" "${fpath}" "${archs}"
-                }
                 default {
-                    ui_debug "unknown file type: ${filetype}"
-                    merge_file "${base}" "${destroot}" "${fpath}" "${archs}"
+                    set filetype [exec [findBinary file $portutil::autoconf::file_path] "-b" "${basepath}${fpath}"]
+                    switch -regexp ${filetype} {
+                        Mach-O.* {
+                            merge_lipo "${base}" "${destroot}" "${fpath}" "${archs}"
+                        }
+                        current\ ar\ archive {
+                            merge_lipo "${base}" "${destroot}" "${fpath}" "${archs}"
+                        }
+                        ASCII\ C\ program\ text {
+                            merge_cpp "${base}" "${destroot}" "${fpath}" "${archs}"
+                        }
+                        default {
+                            ui_debug "unknown file type: ${filetype}"
+                            merge_file "${base}" "${destroot}" "${fpath}" "${archs}"
+                        }
+                    }
                 }
             }
         }
@@ -2325,16 +2362,22 @@ proc chownAsRoot {path} {
 # @param attributes the attributes for the file
 proc fileAttrsAsRoot {file attributes} {
     global euid macportsuser
-    if {[getuid] == 0 && [geteuid] != 0} {
-        # Started as root, but not root now
-        seteuid $euid
-        ui_debug "euid changed to: [geteuid]"
-        ui_debug "setting attributes on $file"
-        eval file attributes {$file} $attributes
-        seteuid [name_to_uid "$macportsuser"]
-        ui_debug "euid changed to: [geteuid]"
+    if {[getuid] == 0} {
+        if {[geteuid] != 0} {
+            # Started as root, but not root now
+            seteuid $euid
+            ui_debug "euid changed to: [geteuid]"
+            ui_debug "setting attributes on $file"
+            eval file attributes {$file} $attributes
+            seteuid [name_to_uid "$macportsuser"]
+            ui_debug "euid changed to: [geteuid]"
+        } else {
+            eval file attributes {$file} $attributes
+        }
     } else {
-        eval file attributes {$file} $attributes
+        # not root, so can't set owner/group
+        set permissions [lindex $attributes [expr [lsearch $attributes "-permissions"] + 1]]
+        file attributes $file -permissions $permissions
     }
 }
 
@@ -2384,3 +2427,168 @@ proc dropPrivileges {} {
     }
 }
 
+# dependency analysis helpers
+
+### _libtest is private; subject to change without notice
+# XXX - Architecture specific
+# XXX - Rely on information from internal defines in cctools/dyld:
+# define DEFAULT_FALLBACK_FRAMEWORK_PATH
+# /Library/Frameworks:/Library/Frameworks:/Network/Library/Frameworks:/System/Library/Frameworks
+# define DEFAULT_FALLBACK_LIBRARY_PATH /lib:/usr/local/lib:/lib:/usr/lib
+#   -- Since /usr/local is bad, using /lib:/usr/lib only.
+# Environment variables DYLD_FRAMEWORK_PATH, DYLD_LIBRARY_PATH,
+# DYLD_FALLBACK_FRAMEWORK_PATH, and DYLD_FALLBACK_LIBRARY_PATH take precedence
+
+proc _libtest {depspec {return_match 0}} {
+    global env prefix frameworks_dir os.platform
+    set depline [lindex [split $depspec :] 1]
+
+    if {[info exists env(DYLD_FRAMEWORK_PATH)]} {
+        lappend search_path $env(DYLD_FRAMEWORK_PATH)
+    } else {
+        lappend search_path ${frameworks_dir} /Library/Frameworks /Network/Library/Frameworks /System/Library/Frameworks
+    }
+    if {[info exists env(DYLD_FALLBACK_FRAMEWORK_PATH)]} {
+        lappend search_path $env(DYLD_FALLBACK_FRAMEWORK_PATH)
+    }
+    if {[info exists env(DYLD_LIBRARY_PATH)]} {
+        lappend search_path $env(DYLD_LIBRARY_PATH)
+    }
+    lappend search_path /lib /usr/lib ${prefix}/lib
+    if {[info exists env(DYLD_FALLBACK_LIBRARY_PATH)]} {
+        lappend search_path $env(DYLD_FALLBACK_LIBRARY_PATH)
+    }
+
+    set i [string first . $depline]
+    if {$i < 0} {set i [string length $depline]}
+    set depname [string range $depline 0 [expr $i - 1]]
+    set depversion [string range $depline $i end]
+    regsub {\.} $depversion {\.} depversion
+    if {${os.platform} == "darwin"} {
+        set depregex \^${depname}${depversion}\\.dylib\$
+    } else {
+        set depregex \^${depname}\\.so${depversion}\$
+    }
+
+    return [_mportsearchpath $depregex $search_path 0 $return_match]
+}
+
+### _bintest is private; subject to change without notice
+
+proc _bintest {depspec {return_match 0}} {
+    global env prefix
+    set depregex [lindex [split $depspec :] 1]
+
+    set search_path [split $env(PATH) :]
+
+    set depregex \^$depregex\$
+
+    return [_mportsearchpath $depregex $search_path 1 $return_match]
+}
+
+### _pathtest is private; subject to change without notice
+
+proc _pathtest {depspec {return_match 0}} {
+    global env prefix
+    set depregex [lindex [split $depspec :] 1]
+
+    # separate directory from regex
+    set fullname $depregex
+
+    regexp {^(.*)/(.*?)$} "$fullname" match search_path depregex
+
+    if {[string index $search_path 0] != "/"} {
+        # Prepend prefix if not an absolute path
+        set search_path "${prefix}/${search_path}"
+    }
+
+    set depregex \^$depregex\$
+
+    return [_mportsearchpath $depregex $search_path 0 $return_match]
+}
+
+# returns the name of the port that will actually be satisfying $depspec
+proc _get_dep_port {depspec} {
+    global registry.installtype
+    set speclist [split $depspec :]
+    set portname [lindex $speclist end]
+    if {[string equal ${registry.installtype} "image"]} {
+        set res [_portnameactive $portname]
+    } else {
+        set res [registry_exists_for_name $portname]
+    }
+    if {$res != 0} {
+        return $portname
+    }
+    
+    set depfile ""
+    switch [lindex $speclist 0] {
+        bin {
+            set depfile [_bintest $depspec 1]
+        }
+        lib {
+            set depfile [_libtest $depspec 1]
+        }
+        path {
+            set depfile [_pathtest $depspec 1]
+        }
+    }
+    if {$depfile == ""} {
+        return $portname
+    } else {
+        set theport [registry_file_registered $depfile]
+        if {$theport != 0} {
+            return $theport
+        } else {
+            return ""
+        }
+    }
+}
+
+# returns the list of archs that the port is targeting
+proc get_canonical_archs {} {
+    global supported_archs os.arch configure.build_arch configure.universal_archs
+    if {$supported_archs == "noarch"} {
+        return "noarch"
+    } elseif {[variant_exists universal] && [variant_isset universal]} {
+        return [lsort -ascii ${configure.universal_archs}]
+    } elseif {${configure.build_arch} != ""} {
+        return ${configure.build_arch}
+    } else {
+        return ${os.arch}
+    }
+}
+
+# check if the installed xcode version is new enough
+proc _check_xcode_version {} {
+    global os.subplatform macosx_version xcodeversion
+
+    if {${os.subplatform} == "macosx"} {
+        switch $macosx_version {
+            10.4 {
+                set min 2.0
+                set ok 2.4.1
+                set rec 2.5
+            }
+            10.5 {
+                set min 3.0
+                set ok 3.1
+                set rec 3.1.4
+            }
+            default {
+                set min 3.2
+                set ok 3.2
+                set rec 3.2.2
+            }
+        }
+        if {$xcodeversion == "none"} {
+            ui_warn "Xcode does not appear to be installed; most ports will likely fail to build."
+        } elseif {[rpm-vercomp $xcodeversion $min] < 0} {
+            ui_error "The installed version of Xcode (${xcodeversion}) is too old to use on the installed OS version. Version $rec or later is recommended on Mac OS X ${macosx_version}."
+            return 1
+        } elseif {[rpm-vercomp $xcodeversion $ok] < 0} {
+            ui_warn "The installed version of Xcode (${xcodeversion}) is known to cause problems. Version $rec or later is recommended on Mac OS X ${macosx_version}."
+        }
+    }
+    return 0
+}
